import unittest
from  ar_corrector.string_manipulator import StringManipulator

class TestStringManipulator(unittest.TestCase):

    def test_get_deletes(self):
        txt = 'cans'
        expected = ['ans', 'cns', 'cas', 'can']
        stringManipulator = StringManipulator(txt)
        res = stringManipulator.get_deletes()
        self.assertEqual(res, expected)
    
    def test_get_switches(self):
        txt = 'eta'
        expected = ['tea', 'eat']
        str_manipulator = StringManipulator(txt)
        res = str_manipulator.get_switches()
        self.assertEqual(res, expected)

    def test_get_replaces(self):
        txt = 'أول'
        expected = 35 * len(txt)
        str_manipulator = StringManipulator(txt)
        res = str_manipulator.get_replaces()
        self.assertEqual(len(res), expected)

    def test_get_inserts(self):
        txt = 'للي'
        expected = 35 * (len(txt) + 1)
        str_manipulator = StringManipulator(txt)
        res = str_manipulator.get_inserts()
        self.assertEqual(len(res), expected)
    
    def test_get_edits1(self):
        txt = 'at'
        expected = 178
        str_manipulator = StringManipulator(txt)
        res = str_manipulator.get_edits1()
        self.assertEqual(len(res), expected)
    
    def test_get_edits2(self):
        txt = 'a'
        expected = 5007
        str_manipulator = StringManipulator(txt)
        res = str_manipulator.get_edits2()
        self.assertEqual(len(res), expected)

if __name__ == '__main__':
    unittest.main()


#%%
import os
import re

alphabet =  'ابتةثجحخدذرزسشصضطظعغفقكلمنهويءآأؤإئ'
vocalizations =  re.sub(r'\s+', r'', 'ْ ِ َ ُ ~ ً ٍ ٌ ّ')
punctuations = re.sub(r'\s+', r'', '، . ؛ ! ؟ :')
allowed_char = alphabet + vocalizations + punctuations

dir_path = os.path.dirname(__file__)
data_dir = dir_path + '/data/'

config = {
    'vocabs_dict': dir_path + '/resources/vocabs.pickle',
    'allowed_char': allowed_char,
    'data_dir': data_dir,

}
if __name__== '__main__':
    print(config)
# %%



#%%

import re
from collections import Counter
from ar_corrector import io_handler
from ar_corrector.proj_config import config
import os

class Preprocessor:
    def __init__(self):
        pass
    def clean(self, txt):
        pattern = r'[^' + re.escape(config["allowed_char"]) + r'\s]+'
        res = re.sub(pattern, r'', txt)
        res = re.sub(r'\s+', r' ', res)
        return res.strip()

    def tokenize(self, txt):
        return txt.split()
    
    def __call__(self, txt):
        ctxt = self.clean(txt)
        txt_arr = self.tokenize(ctxt)
        word_counts = Counter(txt_arr)
        io_handler.save_dict_file(config['vocabs_dict'], dict(word_counts))
        print('vocab_saved')

    def get_letters(self, txt):
        res = self.clean(txt)
        chars = set(res)
        return 
        
    
if __name__ == '__main__':
    text = ''
    data_paths = os.listdir(config['data_dir'])
    data_paths = [config['data_dir'] + dp for dp in data_paths if re.search(r'\.txt$|\.tsv$', dp)]

    for data_path in data_paths:
        text += io_handler.read_txt_file(data_path)
    preprocessor = Preprocessor()
    preprocessor(text)

# %%



class StringManipulator:
    def __init__(self, txt = ''):
        self.splits = [(txt[:i],txt[i:]) for i in range(len(txt) + 1)]
        self.alphbet = sorted('ابتةثجحخدذرزسشصضطظعغفقكلمنهويءآأؤإئ')

    def get_deletes(self):
        deletes = [l + r[1:] for l, r in self.splits if r]
        return deletes
    
    def get_switches(self):
        switches = [l[:-1] + r[0] + l[-1] + r[1:] for l, r in self.splits if r and l]
        return switches
    
    def get_replaces(self):
        replaces = [l + char + r[1:] for l, r in self.splits if r for char in self.alphbet]
        return replaces
    
    def get_inserts(self):
        inserts = [l + char + r for l, r in self.splits for char in self.alphbet]
        return inserts
    
    def get_edits1(self, allow_switches = True):
        deletes = self.get_deletes()
        replaces = self.get_replaces()
        inserts = self.get_inserts()
        edits = deletes + replaces + inserts
        if allow_switches:
            edits += self.get_switches()
        return list(set(edits))
    
    def get_edits2(self, allow_switchs = True):
        edits1 = self.get_edits1(allow_switches=allow_switchs)
        edits2 = set()
        for edit1 in edits1:
            manipulator = StringManipulator(edit1)
            for edit2 in manipulator.get_edits1():
                edits2.add(edit2)
        return list(edits2)
        


import unittest
from ar_corrector.preprocess import Preprocessor

class TestPreprocessor(unittest.TestCase):  

    def test_clean(self):
        preprocessor = Preprocessor()
        txt = '-وهًي ونقية،! و و(- أفضل43 لّ-   {أي  وللمج+تم{ع.1- الًتخص'
        expected = 'وهًي ونقية،! و و أفضل لّ أي وللمجتمع. الًتخص'
        res = preprocessor.clean(txt)
        self.assertEqual(res, expected)
    
    def test_tokenize(self):
        preprocessor = Preprocessor()
        txt = 'بشسم بشم بتشم'
        res = preprocessor.tokenize(txt)
        expected = ['بشسم','بشم' ,'بتشم']
        self.assertEqual(res, expected)
        
        
if __name__ == '__main__':
    unittest.main()


import pickle
import requests
import zipfile
import os
import re
from ar_corrector.proj_config import config

def read_txt_file(path):
    with open(path, 'r', encoding='utf-8-sig') as myfile:
        res = myfile.read()
    return res

def save_dict_file(path, dict_obj):
    with open(path, 'wb') as myfile:
        pickle.dump(dict_obj, myfile)

def load_dict_file(path):
    with open(path, 'rb') as myfile:
        dict_obj = pickle.load(myfile)
        return dict_obj

def download_url(url, save_path, chunk_size=128):
    r = requests.get(url, stream=True)
    with open(save_path, 'wb') as fd:
        for chunk in r.iter_content(chunk_size=chunk_size):
            fd.write(chunk)

def extract_file(path, data_dir):
    with zipfile.ZipFile(path) as my_file:
        print(my_file.namelist())
        my_file.extractall(path=data_dir)

if __name__ == '__main__':
    urls = []
    urls.append('http://www.alcsearch.com/ALCfiles/Download/ALC_in_one/TXT_No_header.txt.zip')
    urls.append('https://raw.githubusercontent.com/mohamedadaly/LABR/master/data/reviews.tsv')
    data_dir = config['data_dir']
    if not os.path.exists(data_dir):
        os.mkdir(data_dir)
    for url in urls:
        file_name = re.split(r'/', url)[-1]
        file_path = os.path.join(data_dir, file_name)
        if not os.path.exists(file_path):
            download_url(url, file_path)
        sufix =  re.search(r'\.zip$', file_name)
        if sufix:
            nfile_name = re.sub(r'\.zip', '', file_name)
            nfile_path = os.path.join(data_dir, nfile_name)
            if not os.path.exists(nfile_path):
                extract_file(file_path, data_dir)    


from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')
setup(
    name='ar_corrector',  # Required
    packages=['ar_corrector'],
    version='v1.1.1',  # Required
    description='Arabic Spelling Correction',  # Optional
    long_description=long_description,  # Optional
    long_description_content_type='text/markdown',  # Optional (see note above)
    download_url = 'https://github.com/basselkassem/ar_corrector/archive/refs/tags/v1.0.0-alpha.tar.gz', 
    author='Bassel Kassem',  # Optional
    author_email='bassel.kassem.job@gmail.com',  # Optional
    classifiers=[
        'Development Status :: 5 - Production/Stable',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',      # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='NLP, Spellingcheck',  # Optional
    python_requires='>=3.6, <4',
    install_requires=['requests'],  # Optional
    package_data={  # Optional
        '': ['resources/*.pickle'],
    },
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/basselkassem/ar_corrector/issues',
        'Funding': 'https://donate.pypi.org',
        'Say Thanks!': 'https://github.com/basselkassem',
        'Source': 'https://github.com/basselkassem/ar_corrector',
    },
)


import unittest
import os
from ar_corrector import io_handler
from ar_corrector.proj_config import config
import re
class TestIoHandler(unittest.TestCase):
    
    def test_read_txt_file(self):
        data_path = os.listdir(config['data_dir'])
        path = [config['data_dir'] + dp for dp in data_path if re.search(r'\.txt$', dp)][0]
        res = io_handler.read_txt_file(path)
        self.assertIsNotNone(res)

    def test_read_text_file_content(self):
        data_path = os.listdir(config['data_dir'])
        path = [config['data_dir'] + dp for dp in data_path if re.search(r'\.txt$', dp)][0]

        res = io_handler.read_txt_file(path)
        self.assertEqual(res[:6], 'الرحلة')

if __name__ == '__main__':
    unittest.main()


#%%
from ar_corrector import io_handler
from ar_corrector.proj_config import config
from ar_corrector.string_manipulator import StringManipulator

class Corrector:
    
    def __init__(self):
        self.vocabs = io_handler.load_dict_file(config['vocabs_dict'])
    
    def is_known(self, txt):
        return txt in self.vocabs.keys()
    
    def filter_edits1(self, txt, num = 1):
        str_manipulator = StringManipulator(txt)
        edits = str_manipulator.get_edits1()
        edits = [(edit, self.vocabs[edit]) for edit in edits if self.is_known(edit)]
        return self.get_most_likely_edit(edits, num)
    
    def filter_edits2(self, txt, num = 1):
        str_manipulator = StringManipulator(txt)
        edits = str_manipulator.get_edits2()
        edits = [(edit, self.vocabs[edit]) for edit in edits if self.is_known(edit)]
        return self.get_most_likely_edit(edits, num)
    
    def check(self, txt):
        ctxt = txt.strip()
        return ctxt in self.vocabs
    
    def get_most_likely_edit(self, edits, num = 1):
        return sorted(edits, key = lambda item: item[1], reverse=True)[:num]

    def __call__(self, txt, num = 1):
        return self.is_known(txt) or self.filter_edits1(txt, num) or self.filter_edits2(txt, num) or txt

if __name__ == '__main__':
    corr = Corrector()
    print(corr('لايعه'))
    print(corr('لوين'))
# %%



import unittest
from ar_corrector.corrector import Corrector

class TestCorrector(unittest.TestCase):
    
    def test_get_most_likely_edit_case1(self):
        edits = [('edit', 1), ('ma', 10), ('df', 20)]
        corr = Corrector()
        expected = [('df', 20)]
        res = corr.get_most_likely_edit(edits)
        self.assertEqual(res, expected)
    
    def test_get_most_likely_edit_case2(self):
        edits = [('edit', 10), ('ma', 5), ('df', 2)]
        corr = Corrector()
        expected = [('edit', 10)]
        res = corr.get_most_likely_edit(edits)
        self.assertEqual(res, expected)

    def test_get_most_likely_edit_case3(self):
        edits = [('edit', 10), ('ma', 50), ('df', 20)]
        corr = Corrector()
        expected = [('ma', 50)]
        res = corr.get_most_likely_edit(edits)
        self.assertEqual(res, expected)

    def test_check_exist(self):
        txt = 'من'
        corr = Corrector()
        res = corr.check(txt)
        self.assertTrue(res)
        
    def test_check_not_exist(self):
        txt = 'بتسمتبمشسك'
        corr = Corrector()
        res = corr.check(txt)
        self.assertFalse(res)

if __name__ == '__main__':
    unittest.main()